## date_gui - Program creates a user interface 
##             that allows the system time, system date,
##             time zone, and ntpd configuration to be easily set
## Copyright (C) 2001, 2002, 2003 Red Hat, Inc.
## Copyright (C) 2001, 2002, 2003 Brent Fox <bfox@redhat.com>
##                                Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gobject
import time
import string
import socket
import mainWindow
import Clock
##
## I18N
## 
from rhpl.translate import _, N_
import rhpl.translate as translate
translate.textdomain ("system-config-date")

custom_widgets = {"clock_widget_create": Clock.Clock}

class datePage:
    def __init__(self, dateBackend, xml):
        self.xml = xml
        #self.clock = Clock.Clock ()
        self.dateBackend = dateBackend

        self.ntpServerChoices = self.dateBackend.readServerChoicesFile ()
        for i in range (0, len (self.ntpServerChoices)):
            host = self.ntpServerChoices[i]
            try:
                socket.inet_aton (host)
                self.ntpServerChoices[i] = socket.gethostbyaddr (host)[0]
            except:
                pass


        self.flag = gtk.FALSE
        times = mainWindow.dateBackend.getDate()

        self.date_frame = self.xml.get_widget ("date_frame")
        self.time_frame = self.xml.get_widget ("time_frame")

        #-----------------Calendar----------#
        self.cal = self.xml.get_widget ("calendar")
        self.cal.select_month(times[1]-1, times[0])
        self.cal.select_day(times[2])

        #-----------------Clock-------------#
        self.clock = self.xml.get_widget ("clock")
        gtk.timeout_add(1000, self.clock.updateTime)

        self.hourEntry = self.xml.get_widget ("hour_entry")
        self.hourEntry.connect("insert-text", self.filter)
        hourAdj = gtk.Adjustment(times[3], 0, 24, 1)
        self.hourEntry.set_adjustment(hourAdj)
        self.hourEntry.set_value(times[3])

        self.minEntry = self.xml.get_widget ("min_entry")
        self.minEntry.connect("insert-text", self.filter)
        minAdj = gtk.Adjustment(times[4], 0, 59, 1)
        self.minEntry.set_adjustment(minAdj)
        self.minEntry.set_value(times[4])

        self.secEntry = self.xml.get_widget ("sec_entry")
        self.secEntry.connect("insert-text", self.filter)
        secAdj = gtk.Adjustment(times[5], 0, 60, 1)
        self.secEntry.set_adjustment(secAdj)
        self.secEntry.set_value(times[5])

        #----------------Timeserver frame---#
        self.ntpFrame = self.xml.get_widget ("ntp_frame")
        
        self.ntpLabel = self.xml.get_widget ("ntp_label")
        
        self.ntpCheckButton = self.xml.get_widget ("ntp_check")
        self.ntpAdvOptionsExpander = self.xml.get_widget ("ntp_adv_options_expander")
        self.ntpAdvOptionsExpander.connect_after ("activate", self.ntpAdvOptionsExpanderActivated)
        self.ntpAdvOptionsExpanderLabel = self.xml.get_widget ("ntp_adv_options_expander_label")
        self.ntpLocalTimeSourceButton = self.xml.get_widget ("ntp_local_timesource_check")
        self.ntpBroadcastClientCheckButton = self.xml.get_widget ("ntp_broadcast_check")
        
        self.srv_frame = self.xml.get_widget ("srv_frame")
        self.ntpComboBoxEntry = self.xml.get_widget ("ntp_comboboxentry")
        self.ntpCombo = self.xml.get_widget ("ntp_combo")
        self.ntpComboEntry = self.xml.get_widget ("ntp_combo_entry")
        if gtk.__dict__.has_key ("ComboBoxEntry"):
            self.ntpCombo.hide ()
            self.ntpCombo = None
            self.ntpComboBoxStore = gtk.ListStore (gobject.TYPE_STRING)
            self.ntpComboBoxEntry.set_model (self.ntpComboBoxStore)
            self.ntpComboBoxEntry.show ()
            self.ntpComboEntry = self.ntpComboBoxEntry.child
        else:
            self.ntpComboBoxEntry.hide ()
            self.ntpComboBoxEntry = None
            self.ntpCombo.show ()
        self.ntpComboEntry.connect ("changed", self.ntpComboBoxEntryChanged)

        self.ntpSrvAddButton = self.xml.get_widget ("ntp_srv_add_button")
        self.ntpSrvPropButton = self.xml.get_widget ("ntp_srv_prop_button")
        self.ntpSrvDelButton = self.xml.get_widget ("ntp_srv_del_button")

        self.ntpListStore = gtk.ListStore (gobject.TYPE_STRING)
        self.ntpServersTreeView = self.xml.get_widget ("ntp_servers_view")
        self.ntpServersTreeView.set_model (self.ntpListStore)
        self.ntpServersTreeView.columns_autosize ()
        col = gtk.TreeViewColumn (_("Server"), gtk.CellRendererText (), text = 0)
        self.ntpServersTreeView.append_column (col)
        self.ntpServersTreeView.get_selection ().connect ("changed", self.ntpServersTreeViewSelectRow)
        
        self.ntpCheckButton.connect("clicked", self.ntpCheckButtonClicked)
        self.ntpSrvAddButton.connect ("clicked", self.ntpSrvAddButtonClicked)
        self.ntpSrvAddButton.set_sensitive (gtk.FALSE)
        self.ntpSrvPropButton.connect ("clicked", self.ntpSrvAddButtonClicked)
        self.ntpSrvDelButton.connect ("clicked", self.ntpSrvDelButtonClicked)
        self.ntpSrvDelButton.set_sensitive (gtk.FALSE)

        #Find out if ntpd is currently running.  If so, activate checkbox
        ntpStatus = mainWindow.dateBackend.isNtpRunning()
        if ntpStatus == 1:
            self.ntpCheckButton.set_active(gtk.TRUE)
        elif ntpStatus == None:
            text = (_("The NTP initscript (%s) does not seem to be functioning "
                      "properly.  Try running 'rpm -V ntp' to see if the initscript "
                      "has been modified.  system-config-date will exit now.")
                      % '/etc/rc.d/init.d/ntpd')
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, text)

            dlg.set_title(_("Error"))
            dlg.set_default_size(100, 100)
            dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
            dlg.set_border_width(2)
            dlg.set_modal(gtk.TRUE)
            rc = dlg.run()
            dlg.destroy()

            import os
            os._exit(1)

        ntpBroadcastClient = mainWindow.dateBackend.getNtpBroadcastClient()
        #print "ntpBroadcastClient = " + str (ntpBroadcastClient)
        self.ntpBroadcastClientCheckButton.set_active(ntpBroadcastClient)

        (self.ntpServers, self.ntpLocalTimeSource) = mainWindow.dateBackend.getNtpServers ()

        self.ntpLocalTimeSourceButton.set_active (self.ntpLocalTimeSource)

        self.ntpListStore.clear ()
        for server in self.ntpServers:
            try:
                ntpServer = socket.gethostbyaddr(server)[0]
            except:
                ntpServer = server
                pass
            iter = self.ntpListStore.append ()
            self.ntpListStore.set_value (iter, 0, ntpServer)

        #if ntpServer:
        #    if ntpServer in self.ntpServers:
        #        #If an NTP server is already specified and is already in the list, then sort and set_popdown
        #        self.ntpServers.sort()
        #        self.ntpCombo.set_popdown_strings(self.ntpServers)
        #        self.ntpCombo.entry.set_text(ntpServer)
        #    else:
        #        #If an NTP server is already specified but not in the list, then add it, sort, and set_popdown
        #        self.ntpServers.append(ntpServer)
        #        self.ntpServers.sort()
        #        self.ntpCombo.set_popdown_strings(self.ntpServers)
        #        self.ntpCombo.entry.set_text(ntpServer)
        #else:
        #    #There's no current NTP server, so just present our default list
        #    self.ntpCombo.set_popdown_strings(self.ntpServers)
        #    self.ntpCombo.entry.set_text("")
        #print "Choices:", self.ntpServerChoices

        self.ntpCheckButtonClicked ()
        self.fillNtpServerChoicesCombo ()

        self.hourEntry.connect("changed", self.changed)
        self.minEntry.connect("changed", self.changed)
        self.secEntry.connect("changed", self.changed)

    def filter(self, widget, text, len, pos):
        if len != 1:
            return

        # first character case:
        if not widget.get_text ():
            if (text[0] not in string.digits):
                widget.emit_stop_by_name ("insert-text")

        # everything else:
        if (text[0] not in string.digits):
            widget.emit_stop_by_name ("insert-text")

    def fillNtpServerChoicesCombo (self):
        if self.ntpCombo:
            self.ntpCombo.set_popdown_strings(self.ntpServerChoices)
        else:
            self.ntpComboBoxStore.clear ()
            for server in self.ntpServerChoices:
                iter = self.ntpComboBoxStore.append ()
                self.ntpComboBoxStore.set_value (iter, 0, server)
        self.ntpComboEntry.set_text("")

    def getDate(self):
        return self.cal.get_date()

    def getTime(self):
        if self.flag == gtk.TRUE:
            hour = self.hourEntry.get_text()
            min = self.minEntry.get_text()
            sec = self.secEntry.get_text()

            return hour, min, sec
        else:
            hour, min, sec = self.clock.getTime()
            return hour, min, sec

    def updateSpinButtons(self):
        hour, min, sec = self.clock.getTime()
        self.hourEntry.set_value(int(hour))
        self.minEntry.set_value(int(min))
        self.secEntry.set_value(int(sec))

    def getNtpEnabled (self):
        return self.ntpCheckButton.get_active ()

    def getNtpServers (self):
        return self.ntpServers

    def getNtpBroadcastClient (self):
        return self.ntpBroadcastClientCheckButton.get_active ()

    def getNtpLocalTimeSource (self):
        return self.ntpLocalTimeSourceButton.get_active ()

    def getNtpServerChoices (self):
        return self.ntpServerChoices

    def ntpCheckButtonClicked (self, *args):
        self.ntpBroadcastClientCheckButton.set_sensitive(self.ntpCheckButton.get_active())
        self.ntpLocalTimeSourceButton.set_sensitive(self.ntpCheckButton.get_active())
        self.srv_frame.set_sensitive(self.ntpCheckButton.get_active())
        self.date_frame.set_sensitive(not self.ntpCheckButton.get_active())
        self.time_frame.set_sensitive(not self.ntpCheckButton.get_active())

    def ntpSrvList (self):
        list = []
        iter = self.ntpListStore.get_iter_first ()
        while iter:
            list.append (self.ntpListStore.get_value (iter, 0))
            iter = self.ntpListStore.iter_next (iter)
        return list

    def ntpSrvValidAdd (self, host):
        return len (host) > 0 and host not in self.ntpSrvList ()

    def ntpAdvOptionsExpanderActivated (self, *args, **kargs):
        if self.ntpAdvOptionsExpander.get_expanded ():
            self.ntpAdvOptionsExpanderLabel.set_label ("_Hide advanced options")
        else:
            self.ntpAdvOptionsExpanderLabel.set_label ("_Show advanced options")

    def ntpComboBoxEntryChanged (self, widget, *args):
        host = self.ntpComboEntry.get_text ()
        self.ntpSrvAddButton.set_sensitive (self.ntpSrvValidAdd (host))

    def ntpServersTreeViewSelectRow (self, *args):
        store, iter = self.ntpServersTreeView.get_selection ().get_selected ()
        if iter:
            self.ntpSrvPropButton.set_sensitive (gtk.TRUE)
            self.ntpSrvDelButton.set_sensitive (gtk.TRUE)
        else:
            self.ntpSrvPropButton.set_sensitive (gtk.FALSE)
            self.ntpSrvDelButton.set_sensitive (gtk.FALSE)

    def ntpSrvAddButtonClicked (self, args):
        host = self.ntpComboEntry.get_text ()
        if self.ntpSrvValidAdd (host):
            self.ntpServers.append (host)
            iter = self.ntpListStore.append ()
            self.ntpListStore.set_value (iter, 0, host)
            if host not in self.ntpServerChoices:
                self.ntpServerChoices.append (host)
            self.fillNtpServerChoicesCombo ()
        
    def ntpSrvPropButtonClicked (self, args):
        pass
        
    def ntpSrvDelButtonClicked (self, args):
        store, iter = self.ntpServersTreeView.get_selection ().get_selected ()
        if iter != None:
            host = self.ntpListStore.get_value (iter, 0)
            self.ntpListStore.remove (iter)
            self.ntpServers.remove (host)
        
    def changed(self, *args):
        self.flag = gtk.TRUE

    def getVBox(self):
        return self.mainVBox

# vim: et ts=4
